"use strict";
const log = require("fary-log");
const Koa = require("koa");
const koaCompress = require("koa-compress");
const koaLogger = require("koa-logger");
const errorMiddleware = require("./middlewares/error");
const proxyMiddleware = require("./middlewares/proxy");
const checkProxy = require("./utils/check-proxy");
const ENV = require("./utils/env");
const serverConfig = require("./utils/server-config");
const { IS_DEV } = ENV;
const logger = log('fary:app');
const DEFAULT_OPTIONS = {
    middlewares: [],
    skipListen: false,
};
/**
 * createApp
 * @param {Object} options - options
 * @param {Function} onStart - callback after server start
 * @return {Koa} app - serverApp
 */
// tslint:disable-next-line:no-empty
function createApp(options, onStart = () => { }) {
    options = Object.assign({}, DEFAULT_OPTIONS, options);
    const proxyConfig = options.proxy || serverConfig.value.proxy;
    checkProxy(proxyConfig);
    logger.log(`Debug mode:`, IS_DEV);
    logger.log(`Proxy mapping:`, proxyConfig);
    const app = new Koa();
    // trust proxy
    app.proxy = true;
    const middlewares = [
        // request/response log
        koaLogger(),
        // compress the response
        koaCompress(),
        // error handler
        errorMiddleware(),
        ...options.middlewares,
        // proxy handler
        proxyMiddleware(),
    ];
    middlewares.forEach((middleware) => {
        if (!middleware) {
            return;
        }
        app.use(middleware);
    });
    app.on('error', (err) => {
        logger.error(`Server error: \n ${err.stack}`);
    });
    if (!options.skipListen) {
        const port = options.port || serverConfig.value.port;
        app.listen(port, () => {
            logger.info(`Koa server is listening at ${port}`);
            onStart();
        });
    }
    process.on('SIGINT', () => {
        process.exit();
    });
    return app;
}
module.exports = createApp;
//# sourceMappingURL=index.js.map